@@ -7,12 +7,17 @@ def get_user_ldap_data(name):
         "creation": datetime(2020, 3, 9, 10, 32, 3, tzinfo=timezone.utc),
         "givenname": "",
         "gpgkeyids": None,
+        "sshpubkeys": None,
         "locked": False,
         "username": name,
         "emails": [f"{name}@example.test"],
         "surname": name,
         "human_name": name,
         "is_private": False,
+        "github_username": name,
+        "gitlab_username": name,
+        "website": f"{name}.example.com",
+        "pronouns": ["they/them/theirs"],
     }
 
 
